!---------------------------------------------------------------------------------------
! FLEXINVERT: read_orog
!---------------------------------------------------------------------------------------
!  FLEXINVERT is free software: you can redistribute it and/or modify
!  it under the terms of the GNU General Public License as published by
!  the Free Software Foundation, either version 3 of the License, or
!  (at your option) any later version.
!
!  FLEXINVERT is distributed in the hope that it will be useful,
!  but WITHOUT ANY WARRANTY; without even the implied warranty of
!  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!  GNU General Public License for more details.
!
!  You should have received a copy of the GNU General Public License
!  along with FLEXINVERT.  If not, see <http://www.gnu.org/licenses/>.
!
!  Copyright 2017, Rona Thompson
!---------------------------------------------------------------------------------------
!
!> read_orog
!!
!! Purpose:    Reads the orography data from file.
!!
!---------------------------------------------------------------------------------------

subroutine read_orog(files)

  use mod_var
  use mod_settings
  use mod_tools,   only : sort
  use mod_save,    only : check
  use netcdf

  implicit none

  type (files_t),  intent(in)             :: files

  character(len=max_path_len)             :: filename
  character(len=max_name_len)             :: varname, lonname, latname
  logical                                 :: lexist
  integer                                 :: nlon, nlat
  integer                                 :: ncid, xid, yid, varid
  integer                                 :: ierr
  real, dimension(:), allocatable         :: lon, lat
  real(kind=8), dimension(:), allocatable :: lono, lato
  integer, dimension(:), allocatable      :: indx, indy

  filename = files%file_orog
  varname = files%varname_orog
  lonname = files%lonname_orog
  latname = files%latname_orog

  inquire(file=trim(filename),exist=lexist)
  if (.not.lexist) then
    write(logid,*) 'ERROR: cannot find '//trim(filename)
    stop
  endif

  ! read file
  ! ---------

  write(logid,*) 'Reading file: '//trim(filename)
  print*, 'lonname, latname, varname = ',lonname,latname,varname

  call check( nf90_open(trim(filename),nf90_NOWRITE,ncid) )
  call check( nf90_inq_dimid(ncid,trim(lonname),xid) )
  call check( nf90_inq_dimid(ncid,trim(latname),yid) )
  call check( nf90_inquire_dimension(ncid,xid,len=nlon) )
  call check( nf90_inquire_dimension(ncid,yid,len=nlat) )

  ! check file resolution matches flexpart
  if ( nlon.ne.nxgrid.or.nlat.ne.nygrid ) then 
    write(logid,*) 'ERROR read_orog: orography file resolution does not match flexpart'
    stop
  endif

  allocate ( lon(nlon) )
  allocate ( lat(nlat) )
  allocate ( indx(nlon) )
  allocate ( indy(nlat) )
  allocate ( orog(nxgrid,nygrid), stat=ierr )
  if ( ierr.ne.0 ) stop 'ERROR read_orog: not enough memory'

  call check( nf90_inq_varid(ncid,trim(lonname),varid) )
  call check( nf90_get_var(ncid,varid,lon) )
  call check( nf90_inq_varid(ncid,trim(latname),varid) )
  call check( nf90_get_var(ncid,varid,lat) )
  call check( nf90_inq_varid(ncid,trim(varname),varid) )
  call check( nf90_get_var(ncid,varid,orog,start=(/1,1,1/),count=(/nxgrid,nygrid,1/)) )
  call check( nf90_close(ncid) )

  allocate ( lono(nlon) )
  allocate ( lato(nlat) )
  lono = dble(lon)
  lato = dble(lat)
  where ( lon.gt.180. ) lon = lon - 360.
  call sort(nlon, lono, indx)
  call sort(nlat, lato, indy)
  orog = orog(indx,indy)

  deallocate(lon)
  deallocate(lat)
  deallocate(lono)
  deallocate(lato)
  deallocate(indx)
  deallocate(indy)

end subroutine read_orog

